"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var SelectboxAnchor_1 = require("./SelectboxAnchor");
var Popover_1 = require("../Popover/Popover");
var Menu_1 = require("../Menu/Menu");
var pure_1 = require("../../utils/pure");
var classnames = require("classnames");
var withTheme_1 = require("../../utils/withTheme");
var ResizeDetector_1 = require("../ResizeDetector/ResizeDetector");
var react_dom_1 = require("react-dom");
var raf_1 = require("@devexperts/utils/dist/function/raf");
exports.SELECTBOX = Symbol('Selectbox');
var RawSelectbox = /** @class */function (_super) {
    tslib_1.__extends(RawSelectbox, _super);
    function RawSelectbox() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isOpened: false
        };
        _this.wrapItem = function (child) {
            var _a = _this.props,
                theme = _a.theme,
                selectedIcon = _a.selectedIcon;
            var value = child.props.value;
            var isActive = typeof value !== 'undefined' && value === _this.props.value;
            var props = Object.assign({}, child.props, {
                isActive: isActive,
                text: child.props.children,
                children: React.createElement("div", { className: theme.container__item }, React.createElement("div", { className: theme.container__item__text }, child.props.children), isActive && React.createElement("span", { className: theme.container__item__activeIcon }, selectedIcon))
            });
            return React.cloneElement(child, props);
        };
        _this.onAnchorClick = function () {
            _this.setState({
                isOpened: !_this.state.isOpened
            });
        };
        _this.onItemSelect = function (value) {
            _this.setState({
                isOpened: false
            });
            _this.props.onValueChange && _this.props.onValueChange(value);
        };
        _this.onPopoverRequestClose = function () {
            _this.setState({
                isOpened: false
            });
        };
        _this.handleAnchorResize = raf_1.raf(function (element) {
            var width = element.getBoundingClientRect().width;
            if (_this.state.width !== width) {
                _this.setState({
                    width: width
                });
            }
        });
        return _this;
    }
    Object.defineProperty(RawSelectbox.prototype, "anchor", {
        get: function get() {
            return this._anchor;
        },
        set: function set(value) {
            if (this._anchor !== value) {
                //uninstall previous
                if (this.anchor && this.props.shouldSyncWidth) {
                    var element = react_dom_1.findDOMNode(this.anchor);
                    ResizeDetector_1.NativeResizeDetector.uninstall(element);
                }
                //install new
                this._anchor = value;
                if (this.anchor && this.props.shouldSyncWidth) {
                    var element = react_dom_1.findDOMNode(this.anchor);
                    ResizeDetector_1.NativeResizeDetector.listenTo(element, this.handleAnchorResize);
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    RawSelectbox.prototype.componentDidMount = function () {
        // if (this.anchor && this.props.shouldSyncWidth) {
        // 	const element = findDOMNode(this.anchor);
        // 	NativeResizeDetector.listenTo(element, this.handleAnchorResize);
        // }
    };
    RawSelectbox.prototype.componentDidUpdate = function (newProps) {
        //todo: rebind
    };
    RawSelectbox.prototype.componentWillUnmount = function () {
        if (this.anchor) {
            var element = react_dom_1.findDOMNode(this.anchor);
            ResizeDetector_1.NativeResizeDetector.uninstall(element);
        }
    };
    RawSelectbox.prototype.render = function () {
        var _this = this;
        var _a = this.props,
            Anchor = _a.Anchor,
            Popover = _a.Popover,
            Menu = _a.Menu,
            placeholder = _a.placeholder,
            children = _a.children,
            caretIcon = _a.caretIcon,
            theme = _a.theme,
            value = _a.value,
            selectedIcon = _a.selectedIcon,
            isDisabled = _a.isDisabled,
            isLoading = _a.isLoading;
        var menuTheme = {
            container: classnames(theme.container__menu, (_b = {}, _b[theme.container__menu_hasSelectedItem] = typeof selectedIcon !== 'undefined' && typeof this.props.value !== 'undefined', _b))
        };
        var anchorTheme = {
            container: theme.container__anchor,
            container_isOpened: theme.container_isOpened,
            text: theme.container__anchor__text,
            content: classnames(theme.container__anchor__content, (_c = {}, _c[theme.container__anchor__content_hasCaret] = !!caretIcon, _c))
        };
        if (caretIcon) {
            anchorTheme.caret = classnames(theme.container__anchor__caret, (_d = {}, _d[theme.container__anchor__caret_isReversed] = this.state.isOpened, _d));
            anchorTheme.wrapperCaret = theme.container__anchor__wrapperCaret;
        }
        var popoverTheme = {
            container: classnames(theme.container__popover),
            content: theme.container__popover__content
        };
        var valueText = placeholder;
        if (typeof value !== 'undefined') {
            var valueChild = React.Children.toArray(children).find(function (child) {
                return child.props.value === value;
            });
            //existance is checked in prop types
            if (valueChild) {
                valueText = valueChild.props.text || valueChild.props.children;
            }
        }
        var popoverStyle;
        if (typeof this.state.width !== 'undefined' && this.props.shouldSyncWidth) {
            popoverStyle = {
                width: this.state.width
            };
        }
        return React.createElement(Anchor, { ref: function ref(el) {
                return _this.anchor = el;
            }, isDisabled: isDisabled, isLoading: isLoading, theme: anchorTheme, caretIcon: caretIcon, isOpened: this.state.isOpened, value: value, valueText: valueText, onClick: this.onAnchorClick }, React.createElement(Popover, { isOpened: this.state.isOpened, style: popoverStyle, theme: popoverTheme, closeOnClickAway: true, onRequestClose: this.onPopoverRequestClose, anchor: this.anchor }, React.createElement(Menu, { onItemSelect: this.onItemSelect, theme: menuTheme }, React.Children.map(children, this.wrapItem))));
        var _b, _c, _d;
    };
    RawSelectbox.propTypes = {
        value: function value(props) {
            var type = _typeof(props.value);
            if (type !== 'undefined') {
                if (type !== 'string' && type !== 'number') {
                    throw new Error('Value should be a string or a number');
                } else {
                    if (!React.Children.toArray(props.children).find(function (child) {
                        return child.props.value === props.value;
                    })) {
                        throw new Error('Value should be in passed children');
                    }
                }
            }
        }
    };
    RawSelectbox.defaultProps = {
        Anchor: SelectboxAnchor_1.SelectboxAnchor,
        Menu: Menu_1.Menu,
        Popover: Popover_1.Popover
    };
    RawSelectbox = tslib_1.__decorate([pure_1.PURE], RawSelectbox);
    return RawSelectbox;
}(React.Component);
exports.Selectbox = withTheme_1.withTheme(exports.SELECTBOX)(RawSelectbox);