'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ThemeProvider = exports.withTheme = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.mergeThemes = mergeThemes;

var _tslib = require('tslib');

var tslib_1 = _interopRequireWildcard(_tslib);

var _react = require('react');

var _propTypes = require('prop-types');

var PropTypes = _interopRequireWildcard(_propTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var THEME_CONTEXT_KEY = '@@dx-util/withTheme-context-key'; //should be serializable
var THEME_CONFIG_KEY = '@@dx-util/withTheme-config-key';
var withTheme = exports.withTheme = function withTheme(name) {
    var defaultTheme = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    function decorate(Target) {
        var _Themed$contextTypes;

        if (Target['config'] && Target['config'].name === name) {
            //already wrapped - just merge in new defaultTheme
            Target['config'].theme = mergeTwo(Target['config'].theme, defaultTheme);
            return Target;
        }
        // noinspection UnnecessaryLocalVariableJS
        var config = {
            name: name,
            theme: defaultTheme
        };

        var Themed = function (_Component) {
            _inherits(Themed, _Component);

            function Themed() {
                _classCallCheck(this, Themed);

                return _possibleConstructorReturn(this, (Themed.__proto__ || Object.getPrototypeOf(Themed)).apply(this, arguments));
            }

            _createClass(Themed, [{
                key: 'render',
                value: function render() {
                    var themr = this.context.themr && this.context.themr.theme && this.context.themr.theme[name];
                    var _a = this.props,
                        withRef = _a.withRef,
                        theme = _a.theme,
                        rest = tslib_1.__rest(_a, ["withRef", "theme"]);
                    var props = Object.assign({}, rest, { ref: withRef, theme: mergeThemes(config.theme, themr, this.context[THEME_CONTEXT_KEY.toString()][name], theme || {}) });
                    return (0, _react.createElement)(Target, props);
                }
            }]);

            return Themed;
        }(_react.Component);

        Themed.displayName = 'Themed(' + (Target.displayName || Target.name) + ')';
        Themed.contextTypes = (_Themed$contextTypes = {}, _defineProperty(_Themed$contextTypes, THEME_CONTEXT_KEY.toString(), PropTypes.object.isRequired), _defineProperty(_Themed$contextTypes, 'themr', PropTypes.shape({
            theme: PropTypes.object.isRequired
        })), _Themed$contextTypes);
        Themed[THEME_CONFIG_KEY] = config;
        return Themed;
    }
    return decorate;
};
/**
 * Merges passed themes by concatenating string keys and processing nested themes
 */
function mergeThemes() {
    for (var _len = arguments.length, themes = Array(_len), _key = 0; _key < _len; _key++) {
        themes[_key] = arguments[_key];
    }

    return themes.reduce(function (acc, theme) {
        return mergeTwo(acc, theme);
    }, {});
}
function mergeTwo() {
    var original = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var mixin = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    //make a copy to avoid mutations of nested objects
    //also strip all functions injected by isomorphic-style-loader
    var result = Object.keys(original).reduce(function (acc, key) {
        var value = original[key];
        if (typeof value !== 'function') {
            acc[key] = value;
        }
        return acc;
    }, {});
    //traverse mixin keys and merge them to resulting theme
    Object.keys(mixin).forEach(function (key) {
        //there's no need to set any defaults here
        var originalValue = result[key];
        var mixinValue = mixin[key];
        switch (typeof mixinValue === 'undefined' ? 'undefined' : _typeof(mixinValue)) {
            case 'object':
                {
                    //possibly nested theme object
                    switch (typeof originalValue === 'undefined' ? 'undefined' : _typeof(originalValue)) {
                        case 'object':
                            {
                                //exactly nested theme object - go recursive
                                result[key] = mergeThemes(originalValue, mixinValue);
                                break;
                            }
                        case 'undefined':
                            {
                                //original does not contain this nested key - just take it as is
                                result[key] = mixinValue;
                                break;
                            }
                        default:
                            {
                                //can't merge an object with a non-object
                                throw new Error('You are merging object ' + key + ' with a non-object ' + originalValue);
                            }
                    }
                    break;
                }
            case 'undefined': //fallthrough - handles accidentally unset values which may come from props
            case 'function':
                {
                    //this handles issue when isomorphic-style-loader addes helper functions to css-module
                    break; //just skip
                }
            default:
                {
                    //plain values
                    switch (typeof originalValue === 'undefined' ? 'undefined' : _typeof(originalValue)) {
                        case 'object':
                            {
                                //can't merge a non-object with an object
                                throw new Error('You are merging non-object ' + mixinValue + ' with an object ' + key);
                            }
                        case 'undefined':
                            {
                                //mixin key is new to original theme - take it as is
                                result[key] = mixinValue;
                                break;
                            }
                        case 'function':
                            {
                                //this handles issue when isomorphic-style-loader addes helper functions to css-module
                                break; //just skip
                            }
                        default:
                            {
                                //finally we can merge
                                result[key] = originalValue.split(' ').concat(mixinValue.split(' ')).filter(function (item, pos, self) {
                                    return self.indexOf(item) === pos && item !== '';
                                }).join(' ');
                                break;
                            }
                    }
                    break;
                }
        }
    });
    return result;
}

var ThemeProvider = exports.ThemeProvider = function (_Component2) {
    _inherits(ThemeProvider, _Component2);

    function ThemeProvider() {
        _classCallCheck(this, ThemeProvider);

        return _possibleConstructorReturn(this, (ThemeProvider.__proto__ || Object.getPrototypeOf(ThemeProvider)).apply(this, arguments));
    }

    _createClass(ThemeProvider, [{
        key: 'render',
        value: function render() {
            return _react.Children.only(this.props.children);
        }
    }, {
        key: 'getChildContext',
        value: function getChildContext() {
            return _defineProperty({}, THEME_CONTEXT_KEY.toString(), this.props.theme);
        }
    }]);

    return ThemeProvider;
}(_react.Component);

ThemeProvider.childContextTypes = _defineProperty({}, THEME_CONTEXT_KEY.toString(), PropTypes.object);